@@ -174,7 +174,107 @@ class GMR(GaussianMixture):
         elif self.rep == 'mode':
             values = weighted_estimated_mean.copy()
         return values
+    
+    def predict_pdf(self, dataset, numbers_of_input_variables, numbers_of_output_variables):
+        """
+        Gaussian Mixture Regression (GMR) based on Gaussian Mixture Model (GMM)
+        
+        Predict values of variables for forward analysis (regression) and inverse analysis. Predition results are given as probability density function
+    
+        Parameters
+        ----------
+        gmm_model: mixture.gaussian_mixture.GaussianMixture
+            GMM model constructed using scikit-learn
+        dataset: numpy.array or pandas.DataFrame
+            (autoscaled) m x n matrix of dataset of training data or test data,
+            m is the number of sammples and
+            n is the number of input variables
+            When this is X-variables, it is forward analysis (regression) and
+            when this is Y-variables, it is inverse analysis
+        numbers_of_input_variables: list
+            vector of numbers of input variables
+            When this is numbers of X-variables, it is forward analysis (regression) and
+            when this is numbers of Y-variables, it is inverse analysis
+        numbers_of_output_variables: list
+            vector of numbers of output variables
+            When this is numbers of Y-variables, it is forward analysis (regression) and
+            when this is numbers of X-variables, it is inverse analysis
+    
+        Returns
+        -------
+        mode_of_estimated_mean : numpy.array
+            (autoscaled) m x k matrix of output variables estimated using mode of weights,
+            k is the number of output variables
+        weighted_estimated_mean : numpy.array
+            (autoscaled) m x k matrix of output variables estimated using weighted mean,
+        estimated_mean_for_all_components : numpy.array
+            (autoscaled) l x m x k matrix of output variables estimated for all components,
+        weights : numpy.array
+            m x l matrix of weights,
+        """
 
+        dataset = np.array(dataset)
+        if dataset.ndim == 0:
+            dataset = np.reshape(dataset, (1, 1))
+        elif dataset.ndim == 1:
+            dataset = np.reshape(dataset, (1, dataset.shape[0]))
+
+        input_means = self.means_[:, numbers_of_input_variables]
+        output_means = self.means_[:, numbers_of_output_variables]
+
+        if self.covariance_type == 'full':
+            all_covariances = self.covariances_
+        elif self.covariance_type == 'diag':
+            all_covariances = np.empty(
+                [self.n_components, self.covariances_.shape[1], self.covariances_.shape[1]])
+            for component_number in range(self.n_components):
+                all_covariances[component_number, :, :] = np.diag(self.covariances_[component_number, :])
+        elif self.covariance_type == 'tied':
+            all_covariances = np.tile(self.covariances_, (self.n_components, 1, 1))
+        elif self.covariance_type == 'spherical':
+            all_covariances = np.empty([self.n_components, self.means_.shape[1], self.means_.shape[1]])
+            for component_number in range(self.n_components):
+                all_covariances[component_number, :, :] = np.diag(
+                    self.covariances_[component_number] * np.ones(self.means_.shape[1]))
+
+        estimated_means = np.zeros([self.n_components, dataset.shape[0], len(numbers_of_output_variables)])
+        estimated_covariances = np.zeros([self.n_components, len(numbers_of_output_variables), len(numbers_of_output_variables)])
+        weights = np.zeros([self.n_components, dataset.shape[0]])
+#        print(all_covariances.shape[2], len(numbers_of_input_variables), len(numbers_of_output_variables))
+        if all_covariances.shape[2] == len(numbers_of_input_variables) + len(numbers_of_output_variables):
+            input_output_covariances = all_covariances[:, numbers_of_input_variables, :]
+            input_covariances = input_output_covariances[:, :, numbers_of_input_variables]
+            input_output_covariances = input_output_covariances[:, :, numbers_of_output_variables]
+            output_input_covariances = all_covariances[:, numbers_of_output_variables, :]
+            output_covariances = output_input_covariances[:, :, numbers_of_output_variables]
+            output_input_covariances = output_input_covariances[:, :, numbers_of_input_variables]
+            
+            # estimated means and weights for all components
+            for component_number in range(self.n_components):
+                estimated_means[component_number, :, :] = output_means[component_number, :] + (
+                        dataset - input_means[component_number, :]).dot(
+                    np.linalg.inv(input_covariances[component_number, :, :])).dot(
+                    input_output_covariances[component_number, :, :])
+                estimated_covariances[component_number, :, :] = output_covariances[component_number, :, :] - output_input_covariances[component_number, :, :].dot(
+                        np.linalg.inv(input_covariances[component_number, :, :])).dot(input_output_covariances[component_number, :, :])
+                weights[component_number, :] = self.weights_[component_number] * \
+                                               multivariate_normal.pdf(dataset,
+                                                                       input_means[component_number, :],
+                                                                       input_covariances[component_number, :, :])
+            if len(np.where(weights.sum(axis=0)==0)[0]) > 0:
+                weights = np.ones(weights.shape)
+            if np.isnan(weights.sum(axis=0)).any():
+                weights = np.ones(weights.shape)
+            if np.isinf(weights.sum(axis=0)).any():
+                weights = np.ones(weights.shape)
+            weights = weights / weights.sum(axis=0)
+
+        else:
+            for component_number in range(self.n_components):
+                estimated_covariances.append(np.zeros([len(numbers_of_output_variables), len(numbers_of_output_variables)]))
+
+        return estimated_means, estimated_covariances, weights
+    
     def cv_opt(self, dataset, numbers_of_input_variables, numbers_of_output_variables, covariance_types,
                numbers_of_components, fold_number):
         """
