@@ -273,3 +273,41 @@ class VBGMR(BayesianGaussianMixture):
         self.rep = max_r2cv_hyperparameter[4]
         self.r2cv = r2cvs[max_r2cv_number]
         
+    def _n_parameters(self):
+        """Return the number of free parameters in the model."""
+        _, n_features = self.means_.shape
+        if self.covariance_type == 'full':
+            cov_params = self.n_components * n_features * (n_features + 1) / 2.
+        elif self.covariance_type == 'diag':
+            cov_params = self.n_components * n_features
+        elif self.covariance_type == 'tied':
+            cov_params = n_features * (n_features + 1) / 2.
+        elif self.covariance_type == 'spherical':
+            cov_params = self.n_components
+        mean_params = n_features * self.n_components
+        return int(cov_params + mean_params + self.n_components - 1)
+    
+    def bic(self, X):
+        """Bayesian information criterion for the current model on the input X.
+        Parameters
+        ----------
+        X : array of shape (n_samples, n_dimensions)
+        Returns
+        -------
+        bic : float
+            The lower the better.
+        """
+        return (-2 * self.score(X) * X.shape[0] +
+                self._n_parameters() * np.log(X.shape[0]))
+
+    def aic(self, X):
+        """Akaike information criterion for the current model on the input X.
+        Parameters
+        ----------
+        X : array of shape (n_samples, n_dimensions)
+        Returns
+        -------
+        aic : float
+            The lower the better.
+        """
+        return -2 * self.score(X) * X.shape[0] + 2 * self._n_parameters()
